@@ -1,3 +1,5 @@
+import { throttle } from 'lodash';
+
 import {
     DOMWidgetModel,
     DOMWidgetView,
@@ -10,6 +12,8 @@ import * as utils from './utils';
 
 import { MODULE_VERSION } from './version';
 
+import { ToolbarView } from './toolbar_widget';
+
 export class MPLCanvasModel extends DOMWidgetModel {
     offscreen_canvas: HTMLCanvasElement;
     offscreen_context: CanvasRenderingContext2D;
@@ -31,13 +35,13 @@ export class MPLCanvasModel extends DOMWidgetModel {
             header_visible: true,
             footer_visible: true,
             toolbar: null,
-            toolbar_visible: true,
+            toolbar_visible: 'fade-in-fade-out',
             toolbar_position: 'horizontal',
             resizable: true,
             capture_scroll: false,
+            pan_zoom_throttle: 33,
             _data_url: null,
-            _width: 0,
-            _height: 0,
+            _size: [0, 0],
             _figure_label: 'Figure',
             _message: '',
             _cursor: 'pointer',
@@ -75,7 +79,7 @@ export class MPLCanvasModel extends DOMWidgetModel {
         this.resize_requested = false;
         this.ratio = (window.devicePixelRatio || 1) / backingStore;
 
-        this.resize_canvas(this.get('_width'), this.get('_height'));
+        this.resize_canvas();
 
         this._init_image();
 
@@ -85,6 +89,10 @@ export class MPLCanvasModel extends DOMWidgetModel {
                 view.update_canvas();
             });
         });
+        this.on('change:_size', () => {
+            this.resize_canvas();
+            this.offscreen_context.drawImage(this.image, 0, 0);
+        });
         this.on('comm_live_update', this.update_disabled.bind(this));
 
         this.update_disabled();
@@ -92,6 +100,10 @@ export class MPLCanvasModel extends DOMWidgetModel {
         this.send_initialization_message();
     }
 
+    get size(): [number, number] {
+        return this.get('_size');
+    }
+
     get disabled(): boolean {
         return !this.comm_live;
     }
@@ -146,13 +158,12 @@ export class MPLCanvasModel extends DOMWidgetModel {
     }
 
     handle_resize(msg: { [index: string]: any }) {
-        const size = msg['size'];
-        this.resize_canvas(size[0], size[1]);
+        this.resize_canvas();
         this.offscreen_context.drawImage(this.image, 0, 0);
 
         if (!this.resize_requested) {
             this._for_each_view((view: MPLCanvasView) => {
-                view.resize_canvas(size[0], size[1]);
+                view.resize_and_update_canvas(this.size);
             });
         }
 
@@ -166,6 +177,9 @@ export class MPLCanvasModel extends DOMWidgetModel {
         }
     }
 
+    /*
+     * Request a resize to the backend
+     */
     resize(width: number, height: number) {
         // Do not request a super small size, as it seems to break the back-end
         if (width <= 5 || height <= 5) {
@@ -174,7 +188,7 @@ export class MPLCanvasModel extends DOMWidgetModel {
 
         this._for_each_view((view: MPLCanvasView) => {
             // Do an initial resize of each view, stretching the old canvas.
-            view.resize_canvas(width, height);
+            view.resize_and_update_canvas([width, height]);
         });
 
         if (this.resize_requested) {
@@ -186,9 +200,12 @@ export class MPLCanvasModel extends DOMWidgetModel {
         }
     }
 
-    resize_canvas(width: number, height: number) {
-        this.offscreen_canvas.width = width * this.ratio;
-        this.offscreen_canvas.height = height * this.ratio;
+    /*
+     * Resize the offscreen canvas
+     */
+    resize_canvas() {
+        this.offscreen_canvas.width = this.size[0] * this.ratio;
+        this.offscreen_canvas.height = this.size[1] * this.ratio;
     }
 
     handle_rubberband(msg: any) {
@@ -272,10 +289,41 @@ export class MPLCanvasModel extends DOMWidgetModel {
         this.image = new Image();
 
         this.image.onload = () => {
+            // In case of an embedded widget, the initial size is not correct
+            // and we are not receiving any resize event from the server
+            if (this.disabled) {
+                this.offscreen_canvas.width = this.image.width;
+                this.offscreen_canvas.height = this.image.height;
+
+                this.offscreen_context.drawImage(this.image, 0, 0);
+
+                this._for_each_view((view: MPLCanvasView) => {
+                    // TODO Make this part of the CanvasView API?
+                    // It feels out of place in the model
+                    view.canvas.width = this.image.width / this.ratio;
+                    view.canvas.height = this.image.height / this.ratio;
+                    view.canvas.style.width = view.canvas.width + 'px';
+                    view.canvas.style.height = view.canvas.height + 'px';
+
+                    view.top_canvas.width = this.image.width / this.ratio;
+                    view.top_canvas.height = this.image.height / this.ratio;
+                    view.top_canvas.style.width = view.top_canvas.width + 'px';
+                    view.top_canvas.style.height =
+                        view.top_canvas.height + 'px';
+
+                    view.canvas_div.style.width = view.canvas.width + 'px';
+                    view.canvas_div.style.height = view.canvas.height + 'px';
+
+                    view.update_canvas(true);
+                });
+
+                return;
+            }
+
+            // Full images could contain transparency (where diff images
+            // almost always do), so we need to clear the canvas so that
+            // there is no ghosting.
             if (this.get('_image_mode') === 'full') {
-                // Full images could contain transparency (where diff images
-                // almost always do), so we need to clear the canvas so that
-                // there is no ghosting.
                 this.offscreen_context.clearRect(
                     0,
                     0,
@@ -283,6 +331,7 @@ export class MPLCanvasModel extends DOMWidgetModel {
                     this.offscreen_canvas.height
                 );
             }
+
             this.offscreen_context.drawImage(this.image, 0, 0);
 
             this._for_each_view((view: MPLCanvasView) => {
@@ -315,7 +364,7 @@ export class MPLCanvasView extends DOMWidgetView {
     canvas_div: HTMLDivElement;
     canvas: HTMLCanvasElement;
     header: HTMLDivElement;
-    toolbar_view: DOMWidgetView;
+    toolbar_view: ToolbarView;
     resize_handle_size: number;
     resizing: boolean;
     context: CanvasRenderingContext2D;
@@ -327,21 +376,20 @@ export class MPLCanvasView extends DOMWidgetView {
     private _resize_event: (event: MouseEvent) => void;
     private _stop_resize_event: () => void;
 
-    render() {
+    async render() {
         this.resizing = false;
         this.resize_handle_size = 20;
 
+        this.el.classList.add('jupyter-matplotlib');
+
         this.figure = document.createElement('div');
-        this.figure.classList.add(
-            'jupyter-matplotlib-figure',
-            'jupyter-widgets',
-            'widget-container',
-            'widget-box',
-            'widget-vbox'
-        );
+        this.figure.classList.add('jupyter-matplotlib-figure');
+
+        this.el.appendChild(this.figure);
 
         this._init_header();
         this._init_canvas();
+        await this._init_toolbar();
         this._init_footer();
 
         this._resize_event = this.resize_event.bind(this);
@@ -349,19 +397,14 @@ export class MPLCanvasView extends DOMWidgetView {
         window.addEventListener('mousemove', this._resize_event);
         window.addEventListener('mouseup', this._stop_resize_event);
 
-        return this.create_child_view(this.model.get('toolbar')).then(
-            (toolbar_view) => {
-                this.toolbar_view = toolbar_view;
+        this.figure.addEventListener('mouseenter', () => {
+            this.toolbar_view.fade_in();
+        });
+        this.figure.addEventListener('mouseleave', () => {
+            this.toolbar_view.fade_out();
+        });
 
-                this._update_toolbar_position();
-
-                this._update_header_visible();
-                this._update_footer_visible();
-                this._update_toolbar_visible();
-
-                this.model_events();
-            }
-        );
+        this.model_events();
     }
 
     model_events() {
@@ -402,64 +445,23 @@ export class MPLCanvasView extends DOMWidgetView {
     }
 
     _update_toolbar_visible() {
-        this.toolbar_view.el.style.display = this.model.get('toolbar_visible')
-            ? ''
-            : 'none';
+        this.toolbar_view.set_visibility(this.model.get('toolbar_visible'));
     }
 
     _update_toolbar_position() {
-        const toolbar_position = this.model.get('toolbar_position');
-        if (toolbar_position === 'top' || toolbar_position === 'bottom') {
-            this.el.classList.add(
-                'jupyter-widgets',
-                'widget-container',
-                'widget-box',
-                'widget-vbox',
-                'jupyter-matplotlib'
-            );
-            this.model.get('toolbar').set('orientation', 'horizontal');
-
-            this.clear();
-
-            if (toolbar_position === 'top') {
-                this.el.appendChild(this.toolbar_view.el);
-                this.el.appendChild(this.figure);
-            } else {
-                this.el.appendChild(this.figure);
-                this.el.appendChild(this.toolbar_view.el);
-            }
-        } else {
-            this.el.classList.add(
-                'jupyter-widgets',
-                'widget-container',
-                'widget-box',
-                'widget-hbox',
-                'jupyter-matplotlib'
-            );
-            this.model.get('toolbar').set('orientation', 'vertical');
-
-            this.clear();
-
-            if (toolbar_position === 'left') {
-                this.el.appendChild(this.toolbar_view.el);
-                this.el.appendChild(this.figure);
-            } else {
-                this.el.appendChild(this.figure);
-                this.el.appendChild(this.toolbar_view.el);
-            }
-        }
-    }
-
-    clear() {
-        while (this.el.firstChild) {
-            this.el.removeChild(this.el.firstChild);
-        }
+        this.model
+            .get('toolbar')
+            .set('position', this.model.get('toolbar_position'));
     }
 
     _init_header() {
         this.header = document.createElement('div');
-        this.header.style.textAlign = 'center';
-        this.header.classList.add('jupyter-widgets', 'widget-label');
+        this.header.classList.add(
+            'jupyter-widgets',
+            'widget-label',
+            'jupyter-matplotlib-header'
+        );
+        this._update_header_visible();
         this._update_figure_label();
         this.figure.appendChild(this.header);
     }
@@ -517,7 +519,10 @@ export class MPLCanvasView extends DOMWidgetView {
         );
         top_canvas.addEventListener(
             'mousemove',
-            this.mouse_event('motion_notify')
+            throttle(
+                this.mouse_event('motion_notify'),
+                this.model.get('pan_zoom_throttle')
+            )
         );
 
         top_canvas.addEventListener(
@@ -529,7 +534,13 @@ export class MPLCanvasView extends DOMWidgetView {
             this.mouse_event('figure_leave')
         );
 
-        top_canvas.addEventListener('wheel', this.mouse_event('scroll'));
+        top_canvas.addEventListener(
+            'wheel',
+            throttle(
+                this.mouse_event('scroll'),
+                this.model.get('pan_zoom_throttle')
+            )
+        );
 
         canvas_div.appendChild(canvas);
         canvas_div.appendChild(top_canvas);
@@ -544,11 +555,24 @@ export class MPLCanvasView extends DOMWidgetView {
             return false;
         });
 
-        this.resize_canvas(this.model.get('_width'), this.model.get('_height'));
-        this.update_canvas();
+        this.resize_and_update_canvas(this.model.size);
     }
 
-    update_canvas() {
+    async _init_toolbar() {
+        this.toolbar_view = (await this.create_child_view(
+            this.model.get('toolbar')
+        )) as ToolbarView;
+
+        this.figure.appendChild(this.toolbar_view.el);
+
+        this._update_toolbar_position();
+        this._update_toolbar_visible();
+    }
+
+    /*
+     * Update the canvas view
+     */
+    update_canvas(stretch = false) {
         if (this.canvas.width === 0 || this.canvas.height === 0) {
             return;
         }
@@ -556,7 +580,18 @@ export class MPLCanvasView extends DOMWidgetView {
         this.top_context.save();
 
         this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
-        this.context.drawImage(this.model.offscreen_canvas, 0, 0);
+
+        if (stretch) {
+            this.context.drawImage(
+                this.model.offscreen_canvas,
+                0,
+                0,
+                this.canvas.width,
+                this.canvas.height
+            );
+        } else {
+            this.context.drawImage(this.model.offscreen_canvas, 0, 0);
+        }
 
         this.top_context.clearRect(
             0,
@@ -629,8 +664,12 @@ export class MPLCanvasView extends DOMWidgetView {
 
     _init_footer() {
         this.footer = document.createElement('div');
-        this.footer.style.textAlign = 'center';
-        this.footer.classList.add('jupyter-widgets', 'widget-label');
+        this.footer.classList.add(
+            'jupyter-widgets',
+            'widget-label',
+            'jupyter-matplotlib-footer'
+        );
+        this._update_footer_visible();
         this._update_message();
         this.figure.appendChild(this.footer);
     }
@@ -639,24 +678,23 @@ export class MPLCanvasView extends DOMWidgetView {
         this.footer.textContent = this.model.get('_message');
     }
 
-    resize_canvas(width: number, height: number) {
+    resize_and_update_canvas(size: [number, number]) {
         // Keep the size of the canvas, and rubber band canvas in sync.
-        this.canvas.setAttribute('width', `${width * this.model.ratio}`);
-        this.canvas.setAttribute('height', `${height * this.model.ratio}`);
-        this.canvas.style.width = width + 'px';
-        this.canvas.style.height = height + 'px';
+        this.canvas.setAttribute('width', `${size[0] * this.model.ratio}`);
+        this.canvas.setAttribute('height', `${size[1] * this.model.ratio}`);
+        this.canvas.style.width = size[0] + 'px';
+        this.canvas.style.height = size[1] + 'px';
 
-        this.top_canvas.setAttribute('width', String(width));
-        this.top_canvas.setAttribute('height', String(height));
+        this.top_canvas.setAttribute('width', String(size[0]));
+        this.top_canvas.setAttribute('height', String(size[1]));
 
-        this.canvas_div.style.width = width + 'px';
-        this.canvas_div.style.height = height + 'px';
+        this.canvas_div.style.width = size[0] + 'px';
+        this.canvas_div.style.height = size[1] + 'px';
 
         this.update_canvas();
     }
 
     mouse_event(name: string) {
-        let last_update = 0;
         return (event: any) => {
             const canvas_pos = utils.get_mouse_position(event, this.top_canvas);
 
@@ -708,22 +746,16 @@ export class MPLCanvasView extends DOMWidgetView {
                 }
             }
 
-            // Rate-limit the position text updates so that we don't overwhelm the
-            // system.
-            if (Date.now() > last_update + 16) {
-                last_update = Date.now();
+            const x = canvas_pos.x * this.model.ratio;
+            const y = canvas_pos.y * this.model.ratio;
 
-                const x = canvas_pos.x * this.model.ratio;
-                const y = canvas_pos.y * this.model.ratio;
-
-                this.model.send_message(name, {
-                    x: x,
-                    y: y,
-                    button: event.button,
-                    step: event.step,
-                    guiEvent: utils.get_simple_keys(event),
-                });
-            }
+            this.model.send_message(name, {
+                x: x,
+                y: y,
+                button: event.button,
+                step: event.step,
+                guiEvent: utils.get_simple_keys(event),
+            });
         };
     }
 
